@@ -183,6 +183,7 @@ sub check_https
 {
 	if (present('protocol') and value('protocol') eq 'https') {
 		if (LWP::UserAgent->is_protocol_supported("https")) {
+			# to get LWP version, use LWP->VERSION instead of LWP::UserAgent->VERSION
 			error('LWP::UserAgent 6.x required to use HTTPS') unless LWP->VERSION() >= 6;
 			require LWP::Protocol::https;
 			error('LWP::Protocol::https 6.x required to use HTTPS') unless LWP::Protocol::https->VERSION && LWP::Protocol::https->VERSION >= 6;
@@ -276,6 +277,7 @@ sub get_config
 			validation(option('secret'), $invalid_format, sub { /^[\x21-\x7e]{40}$/ }),
 			validation(option('region'), $invalid_format, sub { /^[A-Za-z0-9\-]{3,20}$/ }),
 			optional(validation(option('token'), $invalid_format, sub { /^[\x21-\x7e]{20,1024}$/ })),
+			validation(option('timeout', default => 180), $invalid_format, sub { /^[0-9]{1,5}$/ }),
 			validation(option('protocol', default => 'http'), message('protocol must be "https" or "http"'), sub { /^https?$/ }),
 		);
 		
