@@ -20,8 +20,6 @@
 
 package App::MtAws::FileCreateJob;
 
-our $VERSION = '0.975_05';
-
 use strict;
 use warnings;
 use utf8;
@@ -35,14 +33,14 @@ use App::MtAws::Utils;
 
 sub new
 {
-	my ($class, %args) = @_;
-	my $self = \%args;
-	bless $self, $class;
-	defined($self->{filename}) || $self->{stdin} || confess "no filename nor stdin";
-	defined($self->{relfilename}) || confess "no relfilename";
-	$self->{partsize}||die;
-	$self->{raised} = 0;
-	return $self;
+    my ($class, %args) = @_;
+    my $self = \%args;
+    bless $self, $class;
+    defined($self->{filename}||$self->{stdin})||die;
+    defined($self->{relfilename})||die;
+    $self->{partsize}||die;
+    $self->{raised} = 0;
+    return $self;
 }
 
 # returns "ok" "wait" "ok subtask"
@@ -56,7 +54,7 @@ sub get_task
 		
 		if ($self->{stdin}) {
 			$self->{mtime} = time();
-			$self->{fh} = *STDIN;
+		    $self->{fh} = *STDIN;
 		} else {
 			my $binaryfilename = binaryfilename $self->{filename};
 			my $filesize = -s $binaryfilename;
@@ -87,7 +85,6 @@ sub finish_task
 	if ($self->{raised}) {
 		return ("ok replace", App::MtAws::FileUploadJob->new(
 			fh => $self->{fh},
-			finish_cb => $self->{finish_cb},
 			relfilename => $self->{relfilename},
 			partsize => $self->{partsize},
 			upload_id => $task->{result}->{upload_id},
@@ -97,16 +94,4 @@ sub finish_task
 		die;
 	}
 }
-
-sub will_do
-{
-	my ($self) = @_;
-	if (defined($self->{filename})) {
-		"Will UPLOAD $self->{filename}";
-	} elsif ($self->{stdin}) {
-		"Will UPLOAD stream from STDIN";
-	} else {
-		confess;
-	}
-}
-1;
+1;
\ No newline at end of file
