@@ -20,7 +20,6 @@
 
 package App::MtAws::FileFinishJob;
 
-our $VERSION = '0.975_05';
 
 use strict;
 use warnings;
@@ -30,16 +29,16 @@ use base qw/App::MtAws::Job/;
 
 sub new
 {
-	my ($class, %args) = @_;
-	my $self = \%args;
-	bless $self, $class;
-	$self->{upload_id}||die;
-	$self->{filesize}||die;
-	defined($self->{mtime})||die;
-	defined($self->{relfilename})||die;
-	$self->{th}||die;
-	$self->{raised} = 0;
-	return $self;
+    my ($class, %args) = @_;
+    my $self = \%args;
+    bless $self, $class;
+    $self->{upload_id}||die;
+    $self->{filesize}||die;
+    defined($self->{mtime})||die;
+    defined($self->{relfilename})||die;
+    $self->{th}||die;
+    $self->{raised} = 0;
+    return $self;
 }
 
 # returns "ok" "wait" "ok subtask"
@@ -67,14 +66,10 @@ sub finish_task
 {
 	my ($self, $task) = @_;
 	if ($self->{raised}) {
-		if ($self->{finish_cb}) {
-			return ("ok replace", $self->{finish_cb}->($task));
-		} else {
-			return ("done");
-		}
+		return ("done");
 	} else {
 		die;
 	}
 }
 	
-1;
+1;
\ No newline at end of file
