@@ -20,8 +20,6 @@
 
 package App::MtAws::FileListDeleteJob;
 
-our $VERSION = '0.975_05';
-
 use strict;
 use warnings;
 use utf8;
@@ -29,14 +27,14 @@ use base qw/App::MtAws::Job/;
 
 sub new
 {
-	my ($class, %args) = @_;
-	my $self = \%args;
-	bless $self, $class;
-	$self->{archives}||die;
-	$self->{pending}={};
-	$self->{all_raised} = 0;
-	$self->{position} = 0;
-	return $self;
+    my ($class, %args) = @_;
+    my $self = \%args;
+    bless $self, $class;
+    $self->{archives}||die;
+    $self->{pending}={};
+    $self->{all_raised} = 0;
+    $self->{position} = 0;
+    return $self;
 }
 
 # returns "ok" "wait" "ok subtask"
@@ -71,11 +69,5 @@ sub finish_task
 		return ("ok");
 	}
 }
-
-sub will_do
-{
-	my ($self) = @_;
-	map { "Will DELETE archive $_->{archive_id} (filename $_->{relfilename})" } @{$self->{archives}};
-}
 	
-1;
+1;
\ No newline at end of file
