@@ -20,8 +20,6 @@
 
 package App::MtAws::FileUploadJob;
 
-our $VERSION = '0.975_05';
-
 use strict;
 use warnings;
 use utf8;
@@ -32,18 +30,18 @@ use Carp;
 
 sub new
 {
-	my ($class, %args) = @_;
-	my $self = \%args;
-	bless $self, $class;
-	defined($self->{relfilename})||die;
-	$self->{partsize}||die;
-	defined($self->{mtime})||die;
-	$self->{upload_id}||confess;
-	$self->{fh}||die;
-	$self->{all_raised} = 0;
-	$self->{position} = 0;
-	$self->{th} = App::MtAws::TreeHash->new();
-	return $self;
+    my ($class, %args) = @_;
+    my $self = \%args;
+    bless $self, $class;
+    defined($self->{relfilename})||die;
+    $self->{partsize}||die;
+    defined($self->{mtime})||die;
+    $self->{upload_id}||die;
+    $self->{fh}||die;
+    $self->{all_raised} = 0;
+    $self->{position} = 0;
+    $self->{th} = App::MtAws::TreeHash->new();
+    return $self;
 }
 
 # returns "ok" "wait" "ok subtask"
@@ -60,7 +58,7 @@ sub get_task
 			my $part_th = App::MtAws::TreeHash->new(); #TODO: We calc sha twice for same data chunk here
 			$part_th->eat_data(\$data);
 			$part_th->calc_tree();
-
+			
 			my $part_final_hash = $part_th->get_final_hash();
 			my $task = App::MtAws::Task->new(id => $self->{position}, action=>"upload_part", data => {
 				start => $self->{position},
@@ -79,7 +77,7 @@ sub get_task
 			$self->{all_raised} = 1;
 			if (scalar keys %{$self->{uploadparts}} == 0) {
 				# TODO: why do we have to have two FileFinishJob->new ??
-				return ("ok replace", App::MtAws::FileFinishJob->new(finish_cb => $self->{finish_cb}, upload_id => $self->{upload_id}, mtime => $self->{mtime}, filesize => $self->{position}, relfilename => $self->{relfilename}, th => $self->{th}));
+				return ("ok replace", App::MtAws::FileFinishJob->new(upload_id => $self->{upload_id}, mtime => $self->{mtime}, filesize => $self->{position}, relfilename => $self->{relfilename}, th => $self->{th}));
 			} else {
 				return ("wait");
 			}
@@ -93,10 +91,10 @@ sub finish_task
 	my ($self, $task) = @_;
 	delete $self->{uploadparts}->{$task->{id}};
 	if ($self->{all_raised} && scalar keys %{$self->{uploadparts}} == 0) {
-		return ("ok replace", App::MtAws::FileFinishJob->new(finish_cb => $self->{finish_cb}, upload_id => $self->{upload_id}, mtime => $self->{mtime}, relfilename => $self->{relfilename}, filename => $self->{filename}, filesize => $self->{position}, th => $self->{th}));
+		return ("ok replace", App::MtAws::FileFinishJob->new(upload_id => $self->{upload_id}, mtime => $self->{mtime}, relfilename => $self->{relfilename}, filename => $self->{filename}, filesize => $self->{position}, th => $self->{th}));
 	} else {
 		return ("ok");
 	}
 }
-
-1;
+	
+1;
\ No newline at end of file
