@@ -26,11 +26,15 @@ use utf8;
 use POSIX;
 use LWP::UserAgent;
 use HTTP::Request::Common;
-use App::MtAws::TreeHash;
 use Digest::SHA qw/hmac_sha256 hmac_sha256_hex sha256_hex sha256/;
 use App::MtAws::MetaData;
 use App::MtAws::Utils;
 use App::MtAws::Exceptions;
+use App::MtAws::HttpSegmentWriter;
+use Fcntl qw/O_CREAT O_RDWR LOCK_EX LOCK_UN/;
+use File::Temp ();
+use File::Basename;
+use File::Path;
 use Carp;
 
 
@@ -42,10 +46,9 @@ sub new
 	my $self = {};
 	bless $self, $class;
 	
-	defined($self->{$_} = $options->{$_})||confess $_ for (qw/region key secret protocol/);
+	defined($self->{$_} = $options->{$_})||confess $_ for (qw/region key secret protocol timeout/);
 	defined($options->{$_}) and $self->{$_} = $options->{$_} for (qw/vault token/); # TODO: validate vault later
 	
-	
 	confess unless $self->{protocol} =~ /^https?$/; # we check external data here, even if it's verified in the beginning, especially if it's used to construct URL
 	$self->{service} ||= 'glacier';
 	$self->{account_id} = '-';
@@ -213,19 +216,98 @@ sub retrieval_fetch_job
 # TODO: rename
 sub retrieval_download_job
 {
-	my ($self, $jobid, $filename) = @_;
-
+	my ($self, $jobid, $filename, $size, $journal_treehash) = @_;
+	
+	$journal_treehash||confess;
 	$jobid||confess;
 	defined($filename)||confess;
-   
+	$size or confess "no size";
+
 	$self->{url} = "/$self->{account_id}/vaults/$self->{vault}/jobs/$jobid/output";
-	$self->{content_file} = binaryfilename $filename; # TODO: use temp filename for transactional behaviour
+	
+	# TODO: move to ChildWorker?
+	my $dirname = dirname($filename);
+	my $binary_dirname = binaryfilename $dirname;
+	mkpath($binary_dirname);
+	my $tmp = new File::Temp( TEMPLATE => '__mtglacier_temp_XXXXXX', UNLINK => 1, SUFFIX => '.tmp', DIR => $binary_dirname);
+	my $binary_tempfile = $tmp->filename;
+	my $character_tempfile = characterfilename($binary_tempfile);
+	close $tmp;
+	$self->{expected_size} = $size;
+	$self->{writer} = App::MtAws::HttpFileWriter->new(tempfile => $character_tempfile);
+
 	$self->{method} = 'GET';
 
 	my $resp = $self->perform_lwp();
-	return $resp ? 1 : undef; # $resp->decoded_content is undefined here as content_file used
+	my $reported_th = $resp->header('x-amz-sha256-tree-hash') or confess;
+
+	$self->{writer}->treehash->calc_tree();
+	my $th = $self->{writer}->treehash->get_final_hash();
+	
+	$reported_th eq $th or
+		die exception 'treehash_mismatch_full' =>
+		'TreeHash for received file %string filename% (full file) does not match. '.
+		'TreeHash reported by server: %reported%, Calculated TreeHash: %calculated%, TreeHash from Journal: %journal_treehash%',
+		calculated => $th, reported => $reported_th, journal_treehash => $journal_treehash, filename => $filename;
+		# TODO: better report relative filename
+
+	$reported_th eq $journal_treehash or
+		die exception 'treehash_mismatch_journal' =>
+		'TreeHash for received file %string filename% (full file) does not match TreeHash in journal. '.
+		'TreeHash reported by server: %reported%, Calculated TreeHash: %calculated%, TreeHash from Journal: %journal_treehash%',
+		calculated => $th, reported => $reported_th, journal_treehash => $journal_treehash, filename => $filename;
+		# TODO: better report relative filename
+
+	# TODO: move to ChildWorker?
+	$tmp->unlink_on_destroy(0);
+	undef $tmp;
+	rename $binary_tempfile, binaryfilename($filename) or confess "cannot rename file";
+	chmod((0666 & ~umask), binaryfilename($filename)) or confess;
+	
+	return $resp ? 1 : undef;
 }
 
+sub segment_download_job
+{
+	my ($self, $jobid, $tempfile, $filename, $position, $size) = @_;
+
+	$jobid||confess;
+	defined($position) or confess "no position";
+	$size or confess "no size";
+	defined($filename)||confess;
+   
+	$self->{url} = "/$self->{account_id}/vaults/$self->{vault}/jobs/$jobid/output";
+	
+	$self->{expected_size} = $size;
+	$self->{writer} = App::MtAws::HttpSegmentWriter->new(tempfile => $tempfile, position => $position, filename => $filename);
+
+	$self->{method} = 'GET';
+	my $end_position = $position + $size - 1;
+	$self->add_header('Range', "bytes=$position-$end_position");
+
+	my $resp = $self->perform_lwp();
+	$resp && $resp->code == 206 or confess;
+	
+	my $reported_th = $resp->header('x-amz-sha256-tree-hash') or confess;
+	$self->{writer}->treehash->calc_tree();
+	my $th = $self->{writer}->treehash->get_final_hash();
+	
+	$reported_th eq $th or
+		die exception 'treehash_mismatch_segment' =>
+		'TreeHash for received segment of file %string filename% (position %position%, size %size%) does not match. '.
+		'TreeHash reported by server %reported%, Calculated TreeHash %calculated%',
+		calculated => $th, reported => $reported_th, filename => $filename, position => $position, size => $size;
+		# TODO: better report relative filename
+	
+	my ($start, $end, $len) = $resp->header('Content-Range') =~ m!bytes\s+(\d+)\-(\d+)\/(\d+)!;
+
+	confess unless defined($start) && defined($end) && $len;
+	confess unless $end >= $start;
+	confess unless $position == $start;
+	confess unless $end_position == $end;
+	
+	return $resp ? 1 : undef; # $resp->decoded_content is undefined here as content_file used
+}
 
 sub retrieval_download_to_memory
 {
@@ -238,29 +320,8 @@ sub retrieval_download_to_memory
 
 	my $resp = $self->perform_lwp();
 	
-	if ($ENV{MTGLACIER_DEBUG_INVENTORY}) {
-		my $d_content = $resp->decoded_content;
-		my $content = $resp->content;
-		
-		open FF, ">_download_inventory_headers.log";
-		print FF $resp->request->dump;
-		print FF $resp->dump;
-		print FF "\n\nContent length:[".length($d_content)."]\n";
-		print FF "Content match decoded content:[".($d_content eq $content)."]\n";
-		close FF;
-
-		open FF, ">_download_inventory_body.log";
-		print FF $d_content;
-		close FF;
-	}
-
-	if ($resp) {
-		my $r = $resp->decoded_content; # decoded_content is NOT binary string because MIME type is not text/*
-		confess if is_wide_string($r);
-		return $r;
-	} else {
-		return undef;
-	}
+	$resp or confess;
+	return $resp->content;
 }
 
 # TODO: remove
@@ -371,14 +432,34 @@ sub _sign
 }
 
 
+sub _max_retries { 100 }
+sub _sleep($) { sleep shift }
+
+sub throttle
+{
+	my ($i) = @_;
+	if ($i <= 5) {
+		_sleep 1;
+	} elsif ($i <= 10) {
+		_sleep 5;
+	} elsif ($i <= 20) {
+		_sleep 15;
+	} elsif ($i <= 50) {
+		_sleep 60
+	} else {
+		_sleep 180;
+	}
+}
+
 sub perform_lwp
 {
 	my ($self) = @_;
 	
-	for my $i (1..100) {
+	for my $i (1.._max_retries) {
+		undef $self->{last_retry_reason};
 		$self->_sign();
 
-		my $ua = LWP::UserAgent->new(timeout => 120);
+		my $ua = LWP::UserAgent->new(timeout => $self->{timeout});
 		$ua->protocols_allowed ( [ 'https' ] ) if $self->{protocol} eq 'https'; # Lets hard code this.
 		$ua->agent("mt-aws-glacier/$App::MtAws::VERSION (http://mt-aws.com/) "); 
 		my $req = undef;
@@ -413,47 +494,96 @@ sub perform_lwp
 		my $resp = undef;
 
 		my $t0 = time();
-		if ($self->{content_file}) {
+		if ($self->{content_file} && $self->{writer}) {
+			confess "content_file and writer at same time";
+		} elsif ($self->{content_file}) {
 			$resp = $ua->request($req, $self->{content_file});
+		} elsif ($self->{writer}) {
+			my $size = undef;
+			$resp = $ua->request($req, sub {
+				unless (defined($size)) {
+					if ($_[1] && $_[1]->isa('HTTP::Response')) {
+						$size = $_[1]->content_length;
+						if (!$size || ($self->{expected_size} && $size != $self->{expected_size})) {
+							die exception
+								wrong_file_size_in_journal =>
+									'Wrong Content-Length received from server, probably wrong file size in Journal or wrong server';
+						}
+						$self->{writer}->reinit($size);
+					} else {
+						# we should "confess" here, but we cant, only exceptions propogated
+						die exception "unknow_error" => "Unknown error, probably LWP version is too old";
+					}
+				}
+				$self->{writer}->add_data($_[0]);
+				1;
+			});
 		} else {
 			$resp = $ua->request($req);
 		}
 		my $dt = time()-$t0;
 
-		if ($resp->code =~ /^(500|408)$/) {
-			print "PID $$ HTTP ".$resp->code." This might be normal. Will retry ($dt seconds spent for request)\n";
+		if (($resp->code eq '500') && $resp->header('Client-Warning') && ($resp->header('Client-Warning') eq 'Internal response')) { 
+			print "PID $$ HTTP connection problem (timeout?). Will retry ($dt seconds spent for request)\n";
+			$self->{last_retry_reason} = 'Internal response';
 			throttle($i);
-		} elsif (defined($resp->header('X-Died')) && length($resp->header('X-Died')) && $resp->header('X-Died') =~ /^read timeout/i) {
-			print "PID $$ HTTP Timeout. Will retry ($dt seconds spent for request)\n";
+		} elsif ($resp->code =~ /^(500|408)$/) {
+			print "PID $$ HTTP ".$resp->code." This might be normal. Will retry ($dt seconds spent for request)\n";
+			$self->{last_retry_reason} = $resp->code;
+			throttle($i);
+		} elsif (defined($resp->header('X-Died')) && (get_exception($resp->header('X-Died')))) {
+			die $resp->header('X-Died'); # propogate own own exceptions
+		} elsif (defined($resp->header('X-Died')) && length($resp->header('X-Died'))) {
+			print "PID $$ HTTP connection problem. Will retry ($dt seconds spent for request)\n";
+			$self->{last_retry_reason} = 'X-Died';
 			throttle($i);
 		} elsif ($resp->code =~ /^2\d\d$/) {
-			return $resp;
+			if ($self->{writer}) {
+				my ($c, $reason) = $self->{writer}->finish();
+				if ($c eq 'retry') {
+					print "PID $$ HTTP $reason. Will retry ($dt seconds spent for request)\n";
+					$self->{last_retry_reason} = $reason;
+					throttle($i);
+				} elsif ($c ne 'ok') {
+					confess;
+				} else {
+					return $resp;
+				}
+			} elsif (defined($resp->content_length) && $resp->content_length != length($resp->content)){
+				print "PID $$ HTTP Unexpected end of data. Will retry ($dt seconds spent for request)\n";
+				$self->{last_retry_reason}='Unexpected end of data';
+				throttle($i);
+			} else {
+				return $resp;
+			}
 		} else {
+			if ($resp->code =~ /^40[03]$/) {
+				if ($resp->content_type && $resp->content_type eq 'application/json') {
+					my $json = JSON::XS->new->allow_nonref;
+					my $scalar = eval { $json->decode( $resp->content ); }; # we assume content always in utf8
+					if (defined $scalar) {
+						my $code = $scalar->{code};
+						my $type = $scalar->{type};
+						my $message = $scalar->{message};
+						if ($code eq 'ThrottlingException') {
+							print "PID $$ ThrottlingException. Will retry ($dt seconds spent for request)\n";
+							$self->{last_retry_reason} = 'ThrottlingException';
+							throttle($i);
+							next;
+						}
+					}
+				}
+			}
 			print STDERR "Error:\n";
 			print STDERR $req->dump;
 			print STDERR $resp->dump;
 			print STDERR "\n";
-			return undef;
+			die exception 'http_unexpected_reply' => 'Unexpected reply from remote server';
 		}
 	}
-	return undef;
+	die exception 'too_many_tries' => "Request was not successful after "._max_retries." retries";
 }
 
-sub throttle
-{
-	my ($i) = @_;
-	if ($i <= 5) {
-		sleep 1;
-	} elsif ($i <= 10) {
-		sleep 5;
-	} elsif ($i <= 20) {
-		sleep 15;
-	} elsif ($i <= 50) {
-		sleep 60
-	} else {
-		sleep 180;
-	}
-}
 
 
 sub get_signature_key
@@ -479,3 +609,15 @@ sub trim
 
 
 1;
+__END__
+		} elsif ($resp->code =~ /^40[03]$/) {
+			if ($resp->content_type eq 'application/json') {
+				my $json = JSON::XS->new->allow_nonref;
+				my $scalar = eval { $json->decode( $resp->content ); }; # we assume content always in utf8
+				if (defined $scalar) {
+					my $code = $scalar->{code};
+					my $type = $scalar->{type};
+					my $message = $scalar->{message};
+				}
+			}
+			die;
