@@ -20,19 +20,17 @@
 
 package App::MtAws::JobProxy;
 
-our $VERSION = '0.981_01';
-
 use strict;
 use warnings;
 use utf8;
 
 sub new
 {
-	my ($class, %args) = @_;
-	my $self = \%args;
-	$self->{job}||die;
-	bless $self, $class;
-	return $self;
+    my ($class, %args) = @_;
+    my $self = \%args;
+    $self->{job}||die;
+    bless $self, $class;
+    return $self;
 }
 
 # returns "ok" "wait" "ok subtask"
@@ -43,7 +41,7 @@ sub get_task
 	
 	if ($r[0] eq 'ok replace'){
 		$self->{job} = $r[1];
-		@r = $self->{job}->get_task(@a);
+		 @r = $self->{job}->get_task(@a);
 	}
 	return @r;
 }
@@ -60,12 +58,5 @@ sub finish_task
 		return @res;
 	}
 }
-
-sub will_do
-{
-	# TODO: does it make any sense to call will_do() after job replaced?? probably yes..
-	# for example we can dump pending jobs anytime.. 
-	shift->{job}->will_do();
-}
 	
-1;
+1;
\ No newline at end of file
