@@ -20,8 +20,6 @@
 
 package App::MtAws::Journal;
 
-our $VERSION = '0.981_01';
-
 use strict;
 use warnings;
 use utf8;
@@ -35,31 +33,23 @@ use IO::Handle;
 use App::MtAws::Utils;
 use App::MtAws::Exceptions;
 use App::MtAws::Filter;
-use App::MtAws::FileVersions;
 
 sub new
 {
 	my ($class, %args) = @_;
-
-	my %checkargs = %args;
-	exists $checkargs{$_} && delete $checkargs{$_}
-		for qw/root_dir journal_file journal_encoding output_version leaf_optimization use_active_retrievals filter follow/;
-	confess "Unknown argument(s) to Journal constructor: ".join(', ', keys %checkargs) if %checkargs; # TODO: test
-
 	my $self = \%args;
 	bless $self, $class;
-
+	
 	$self->{journal_encoding} ||= 'UTF-8';
-
+	
 	defined($self->{journal_file}) || confess;
 	$self->{journal_h} = {};
-	$self->{archive_h} = {};
-
+	
 	$self->{used_versions} = {};
-	$self->{output_version} = 'B' unless defined($self->{output_version});
-	$self->{last_supported_version} = 'C';
-	$self->{first_unsupported_version} = chr(ord($self->{last_supported_version})+1);
-
+	$self->{output_version} = 'A' unless defined($self->{output_version});
+	$self->{last_supported_version} = 'A';
+	$self->{first_unsupported_version} = chr(ord('A')+1);
+	
 	return $self;
 }
 
@@ -76,7 +66,7 @@ sub read_journal
 	confess unless length($self->{journal_file});
 	$self->{last_read_time} = time();
 	$self->{active_retrievals} = {} if $self->{use_active_retrievals};
-
+	
 	my $binary_filename = binaryfilename $self->{journal_file};
 	if ($args{should_exist} && !-e $binary_filename) {
 		confess;
@@ -86,15 +76,14 @@ sub read_journal
 				filename => $self->{journal_file}, errno => $!;
 		my $lineno = 0;
 		while (!eof($F)) {
-			defined( my $line = <$F> ) or confess;
+			defined( $_ = <$F> ) or confess;
 			++$lineno;
-			$line =~ s/\r?\n$// or
+			s/\r?\n$// or
 				die exception journal_format_error => "Invalid format of journal, line %lineno% not fully written", lineno => $lineno;
-			$self->process_line($line, $lineno);
+			$self->process_line($_, $lineno);
 		}
 		close $F or confess;
 	}
-	$self->_index_archives_as_files();
 	return;
 }
 
@@ -103,8 +92,8 @@ sub open_for_write
 	my ($self) = @_;
 	open_file($self->{append_file}, $self->{journal_file}, mode => '>>', file_encoding => $self->{journal_encoding}) or
 		die exception journal_open_error => "Unable to open journal file %string filename% for writing, errno=%errno%",
-			filename => $self->{journal_file}, errno => $!;
-	$self->{append_file}->autoflush();
+			filename => $self->{journal_file}, errno => $!;	
+  	$self->{append_file}->autoflush();
 }
 
 sub close_for_write
@@ -118,62 +107,44 @@ sub process_line
 {
 	my ($self, $line, $lineno) = @_;
 	try_drop_utf8_flag $line;
-	my ($ver, $time, $archive_id, $size, $mtime, $treehash, $relfilename, $job_id);
+	my ($time, $archive_id, $size, $mtime, $treehash, $relfilename);
 	# TODO: replace \S and \s, make tests for this
 
-	# Journal version 'A', 'B', 'C'
-	# 'B' and 'C' two way compatible
-	# 'A' is not compatible, but share some common code
-	if (($ver, $time, $archive_id, $size, $mtime, $treehash, $relfilename) =
-		$line =~ /^([ABC])\t([0-9]{1,20})\tCREATED\t(\S+)\t([0-9]{1,20})\t([+-]?[0-9]{1,20}|NONE)\t(\S+)\t(.*?)$/) {
-		confess "invalid filename" unless is_relative_filename($relfilename);
-
-		# here goes difference between 'A' and 'B','C'
-		if ($ver eq 'A') {
-			confess if $mtime eq 'NONE'; # this is not supported by format 'A'
-
-			# version 'A' produce records with mtime set even when there is no mtime in Amazon metadata
-			# (this is possible when archive uploaded by 3rd party program, or mtglacier <= v0.7)
-			# we detect this as $archive_id eq $relfilename - this is practical impossible
-			# unless such record was created by download-inventory command
-			$mtime = undef if ($archive_id eq $relfilename);
-		} else {
-			$mtime = undef if $mtime eq 'NONE';
-		}
-
-
-		$self->_add_archive({
-			relfilename => $relfilename,
+		# Journal version 'A'
+	if (($time, $archive_id, $size, $mtime, $treehash, $relfilename) =
+		$line =~ /^A\t([0-9]{1,20})\tCREATED\t(\S+)\t([0-9]{1,20})\t([+-]?[0-9]{1,20})\t(\S+)\t(.*?)$/) {
+		confess "invalid filename" unless defined(is_relative_filename($relfilename));
+		$self->_add_file($relfilename, {
 			time => $time,
 			archive_id => $archive_id,
 			size => $size,
 			mtime => $mtime,
 			treehash => $treehash,
 		});
-		$self->{used_versions}->{$ver} = 1 unless $self->{used_versions}->{$ver};
-	} elsif (($ver, $time, $archive_id, $relfilename) = $line =~ /^([ABC])\t([0-9]{1,20})\tDELETED\t(\S+)\t(.*?)$/) {
-		$self->_delete_archive($archive_id); # TODO avoid stuff like $1 $2 $3 etc
-		$self->{used_versions}->{$ver} = 1 unless $self->{used_versions}->{$ver};
-	} elsif (($ver, $time, $archive_id, $job_id) = $line =~ /^([ABC])\t([0-9]{1,20})\tRETRIEVE_JOB\t(\S+)\t(.*?)$/) {
+		$self->{used_versions}->{A} = 1 unless $self->{used_versions}->{A};
+	} elsif ($line =~ /^A\t([0-9]{1,20})\tDELETED\t(\S+)\t(.*?)$/) {
+		$self->_delete_file($3); # TODO avoid stuff like $1 $2 $3 etc
+		$self->{used_versions}->{A} = 1 unless $self->{used_versions}->{A};
+	} elsif ($line =~ /^A\t([0-9]{1,20})\tRETRIEVE_JOB\t(\S+)\t(.*?)$/) {
+		my ($time, $archive_id, $job_id) = ($1,$2,$3);
 		$self->_retrieve_job($time, $archive_id, $job_id);
-		$self->{used_versions}->{$ver} = 1 unless $self->{used_versions}->{$ver};
-
+		$self->{used_versions}->{A} = 1 unless $self->{used_versions}->{A};
+		
 	# Journal version '0'
-
+	
 	} elsif (($time, $archive_id, $size, $treehash, $relfilename) =
 		$line =~ /^([0-9]{1,20}) CREATED (\S+) ([0-9]{1,20}) (\S+) (.*?)$/) {
-		confess "invalid filename" unless is_relative_filename($relfilename);
-		$self->_add_archive({
-			relfilename => $relfilename,
+		confess "invalid filename" unless defined(is_relative_filename($relfilename));
+		#die if $self->{journal_h}->{$relfilename};
+		$self->_add_file($relfilename, {
 			time => $time,
-			mtime => undef,
 			archive_id => $archive_id,
 			size => $size,
 			treehash => $treehash,
 		});
 		$self->{used_versions}->{0} = 1 unless $self->{used_versions}->{0};
 	} elsif ($line =~ /^[0-9]{1,20}\s+DELETED\s+(\S+)\s+(.*?)$/) { # TODO: delete file, parse time too!
-		$self->_delete_archive($1);
+		$self->_delete_file($2);
 		$self->{used_versions}->{0} = 1 unless $self->{used_versions}->{0};
 	} elsif (($time, $archive_id) = $line =~ /^([0-9]{1,20})\s+RETRIEVE_JOB\s+(\S+)$/) {
 		$self->_retrieve_job($time, $archive_id);
@@ -190,44 +161,18 @@ sub process_line
 	}
 }
 
-sub _add_archive
+sub _add_file
 {
-	my ($self, $args) = @_;
-	if (!$self->{filter} || $self->{filter}->check_filenames($args->{relfilename})) {
-		confess "duplicate entry" if $self->{archive_h}{$args->{archive_id}};
-		$self->{archive_h}{$args->{archive_id}} = $args;
-	}
+	my ($self, $relfilename, $args) = @_;
+	$self->{journal_h}->{$relfilename} = $args if (!$self->{filter} || $self->{filter}->check_filenames($relfilename));
 }
 
-sub _delete_archive
+sub _delete_file
 {
-	my ($self, $archive_id) = @_;
-	$self->{archive_h}{$archive_id} or confess "archive $archive_id not found in archive_h"; # TODO: put it to backlog, process later?
-	delete $self->{archive_h}{$archive_id};
-}
-
-sub _add_filename
-{
-	my ($self, $args) = @_;
-	my $relfilename = $args->{relfilename};
-	if ($self->{journal_h}{$relfilename}) {
-		if (ref $self->{journal_h}{$relfilename} eq ref {}) {
-			my $v = App::MtAws::FileVersions->new();
-			$v->add($self->{journal_h}{$relfilename});
-			$v->add($args);
-			$self->{journal_h}{$relfilename} = $v;
-		} else {
-			$self->{journal_h}{$relfilename}->add($args);
-		}
-	} else {
-		$self->{journal_h}{$relfilename} = $args
-	}
-}
-
-sub _index_archives_as_files
-{
-	my ($self) = @_;
-	$self->_add_filename($_) for (values %{$self->{archive_h}});
+	my ($self, $relfilename) = @_;
+	delete $self->{journal_h}->{$relfilename}
+		if (!$self->{filter} || $self->{filter}->check_filenames($relfilename)) && $self->{journal_h}->{$relfilename};
+		# TODO: exception or warning if $files->{$2}
 }
 
 sub _retrieve_job
@@ -241,13 +186,6 @@ sub _retrieve_job
 	}
 }
 
-sub latest
-{
-	my ($self, $relfilename) = @_;
-	my $e = $self->{journal_h}{$relfilename} or confess "$relfilename not found in journal";
-	(ref $e eq ref {}) ? $e : $e->latest();
-}
-
 #
 # Wrting journal
 #
@@ -255,26 +193,25 @@ sub latest
 sub add_entry
 {
 	my ($self, $e) = @_;
-
-	confess unless $self->{output_version} eq 'B';
-
+	
+	confess unless $self->{output_version} eq 'A';
+	
 	# TODO: time should be ascending?
 
 	if ($e->{type} eq 'CREATED') {
 		#" CREATED $archive_id $data->{filesize} $data->{final_hash} $data->{relfilename}"
-		defined( $e->{$_} ) || confess "bad $_" for (qw/time archive_id size treehash relfilename/);
-		confess "invalid filename" unless is_relative_filename($e->{relfilename});
-		my $mtime = defined($e->{mtime}) ? $e->{mtime} : 'NONE';
-		$self->_write_line("B\t$e->{time}\tCREATED\t$e->{archive_id}\t$e->{size}\t$mtime\t$e->{treehash}\t$e->{relfilename}");
+		defined( $e->{$_} ) || confess "bad $_" for (qw/time archive_id size mtime treehash relfilename/);
+		confess "invalid filename" unless defined(is_relative_filename($e->{relfilename}));
+		$self->_write_line("A\t$e->{time}\tCREATED\t$e->{archive_id}\t$e->{size}\t$e->{mtime}\t$e->{treehash}\t$e->{relfilename}");
 	} elsif ($e->{type} eq 'DELETED') {
 		#  DELETED $data->{archive_id} $data->{relfilename}
 		defined( $e->{$_} ) || confess "bad $_" for (qw/archive_id relfilename/);
-		confess "invalid filename" unless is_relative_filename($e->{relfilename});
-		$self->_write_line("B\t$e->{time}\tDELETED\t$e->{archive_id}\t$e->{relfilename}");
+		confess "invalid filename" unless defined(is_relative_filename($e->{relfilename}));
+		$self->_write_line("A\t$e->{time}\tDELETED\t$e->{archive_id}\t$e->{relfilename}");
 	} elsif ($e->{type} eq 'RETRIEVE_JOB') {
 		#  RETRIEVE_JOB $data->{archive_id}
 		defined( $e->{$_} ) || confess "bad $_" for (qw/archive_id job_id/);
-		$self->_write_line("B\t$e->{time}\tRETRIEVE_JOB\t$e->{archive_id}\t$e->{job_id}");
+		$self->_write_line("A\t$e->{time}\tRETRIEVE_JOB\t$e->{archive_id}\t$e->{job_id}");
 	} else {
 		confess "Unexpected else";
 	}
@@ -292,37 +229,49 @@ sub _write_line
 # Reading file listing
 #
 
-sub read_files
+sub read_all_files
+{
+	my ($self) = @_;
+	$self->{allfiles_a} = $self->_read_files('all');
+}
+
+sub read_new_files
+{
+	my ($self, $max_number_of_files) = @_;
+	$self->{newfiles_a} = $self->_read_files('new', $max_number_of_files);
+}
+
+sub read_existing_files
+{
+	my ($self) = @_;
+	$self->{existingfiles_a} = $self->_read_files('existing');
+}
+
+
+sub _read_files
 {
 	my ($self, $mode, $max_number_of_files) = @_;
-
-	my %checkmode = %$mode;
-	defined $checkmode{$_} && delete $checkmode{$_} for qw/new existing missing/;
-	confess "Unknown mode: ".join(';', keys %checkmode) if %checkmode;
-
+	
 	confess unless defined($self->{root_dir});
-
-	my %missing = $mode->{'missing'} ? %{$self->{journal_h}} : ();
-
-	$self->{listing} = { new => [], existing => [], missing => [] };
+	my $filelist = [];
 	my $i = 0;
 	# TODO: find better workaround than "-s"
 	$File::Find::prune = 0;
 	$File::Find::dont_use_nlink = !$self->{leaf_optimization};
 	File::Find::find({ wanted => sub {
-		if ($self->_listing_exceeed_max_number_of_files($max_number_of_files)) {
+		if ($max_number_of_files && (scalar @$filelist >= $max_number_of_files)) {
 			$File::Find::prune = 1;
 			return;
 		}
-
+		
 		if (++$i % 1000 == 0) {
 			print "Found $i local files\n";
 		}
-
+		
 		# note that this exception is probably thrown even if a directory below transfer root contains invalid chars
 		die exception(invalid_chars_filename => "Not allowed characters in filename: %filename%", filename => hex_dump_string($_))
 			if /[\r\n\t]/;
-
+				
 		if (-d) {
 			my $dir = character_filename($_);
 			my $reldir = File::Spec->abs2rel($dir, $self->{root_dir});
@@ -330,46 +279,25 @@ sub read_files
 				my ($match, $matchsubdirs) = $self->{filter}->check_dir($reldir."/");
 				if (!$match && $matchsubdirs) {
 					$File::Find::prune = 1;
-				}
+				} 
 			}
 		} else {
-			# file can be not existing here (i.e. dangling symlink)
 			my $filename = character_filename(my $binaryfilename = $_);
 			my $orig_relfilename = File::Spec->abs2rel($filename, $self->{root_dir});
 			if (!$self->{filter} || $self->{filter}->check_filenames($orig_relfilename)) {
 				if ($self->_is_file_exists($binaryfilename)) {
-					my $relfilename;
-					confess "Invalid filename: ".hex_dump_string($orig_relfilename)
-						unless defined($relfilename = sanity_relative_filename($orig_relfilename));
-					if (my $use_mode = $self->_can_read_filename_for_mode($orig_relfilename, $mode)) {
-						push @{$self->{listing}{$use_mode}}, { relfilename => $relfilename }; # TODO: we can reduce memory usage even more. we don't need hash here probably??
+					if ($self->_can_read_filename_for_mode($orig_relfilename, $mode)) {
+						my $relfilename;
+						confess "Invalid filename: ".hex_dump_string($orig_relfilename)
+							unless defined($relfilename = sanity_relative_filename($orig_relfilename));
+						push @$filelist, { relfilename => $relfilename }; # TODO: we can reduce memory usage even more. we don't need hash here probably??
 					}
-					delete $missing{$relfilename} if ($mode->{missing});
 				}
 			}
 		}
-	}, no_chdir => 1, $self->{follow} ? (follow => 1, follow_skip => 2) : () }, (binaryfilename($self->{root_dir})));
-
-	if ($mode->{missing} && !$self->_listing_exceeed_max_number_of_files($max_number_of_files)) {
-		for (keys %missing) {
-			unless ($self->_is_file_exists(binaryfilename $self->absfilename($_))) {
-				push @{$self->{listing}{missing}}, { relfilename => $_ };
-				last if $self->_listing_exceeed_max_number_of_files($max_number_of_files);
-			}
-		}
-	}
-}
-
-sub _listing_exceeed_max_number_of_files
-{
-	my ($self, $max_number_of_files) = @_;
-	($max_number_of_files && (
-		(
-			(scalar @{$self->{listing}{new}}) +
-			(scalar @{$self->{listing}{existing}}) +
-			(scalar @{$self->{listing}{missing}})
-		)  >= $max_number_of_files)
-	);
+	}, no_chdir => 1 }, (binaryfilename($self->{root_dir})));
+	
+	$filelist;
 }
 
 sub character_filename
@@ -400,26 +328,23 @@ sub absfilename
 sub _can_read_filename_for_mode
 {
 	my ($self, $relfilename, $mode) = @_;
-
-	if (defined($self->{journal_h}->{$relfilename})) {
-		if ($mode->{existing}) {
-			return 'existing';
-		} elsif ($mode->{new}) { # AND not $mode->{existing}
+	my $ok = 0;
+	if ($mode eq 'all') {
+		$ok = 1;
+	} elsif ($mode eq 'new') {
+		if (!defined($self->{journal_h}->{$relfilename})) {
+			$ok = 1;
+		} else {
 			print "Skip $relfilename\n";
-			return 0;
-		} else {
-			return 0;
 		}
-	} else {
-		if ($mode->{new}) {
-			return 'new';
-		} elsif ($mode->{existing}) { # AND not $mode->{new}
-			print "Not exists $relfilename\n";
-			return 0;
+	} elsif ($mode eq 'existing') {
+		if (defined($self->{journal_h}->{$relfilename})) {
+			$ok = 1;
 		} else {
-			return 0;
+			print "Not exists $relfilename\n";
 		}
 	}
+	$ok;
 }
 
 
