@@ -20,8 +20,6 @@
 
 package App::MtAws::ParentWorker;
 
-our $VERSION = '0.981_01';
-
 use lib 'lib';
 
 use strict;
@@ -31,24 +29,26 @@ use App::MtAws::LineProtocol;
 use Carp;
 use POSIX;
 use App::MtAws::Utils;
+use bytes;
+no bytes;
 
 sub new
 {
-	my ($class, %args) = @_;
-	my $self = \%args;
-	$self->{children}||die;
-	$self->{disp_select}||die;
-	$self->{options}||die;
-	@{$self->{freeworkers}} = keys %{$self->{children}};
-	bless $self, $class;
-	return $self;
+    my ($class, %args) = @_;
+    my $self = \%args;
+    $self->{children}||die;
+    $self->{disp_select}||die;
+    $self->{options}||die;
+    @{$self->{freeworkers}} = keys %{$self->{children}};
+    bless $self, $class;
+    return $self;
 }
 
 sub process_task
 {
 	my ($self, $ft, $journal) = @_;
 	my $task_list = {};
-	while () {
+	while (1) {
 		if ( @{$self->{freeworkers}} ) {
 			my ($result, $task) = $ft->get_task();
 			if ($result eq 'wait') {
@@ -63,8 +63,6 @@ sub process_task
 				$task_list->{$task->{id}} = $task;
 				send_data($worker->{tochild}, $task->{action}, $task->{id}, $task->{data}, $task->{attachment}) or
 					$self->comm_error;
-			} elsif ($result eq 'done') {
-				return (1, undef);
 			} else {
 				die;
 			}
@@ -81,6 +79,11 @@ sub wait_worker
 	my @ready;
 	do { @ready = $self->{disp_select}->can_read(); } until @ready || $! != EINTR;
 	for my $fh (@ready) {
+		#if (eof($fh)) {
+		#	$self->{disp_select}->remove($fh);
+		#	die "Unexpeced EOF in Pipe";
+		#	next; 
+		#}
 		my ($pid, undef, $taskid, $data, $resultattachmentref) = get_data($fh);
 		$pid or $self->comm_error;
 		push @{$self->{freeworkers}}, $pid;
@@ -88,17 +91,17 @@ sub wait_worker
 		$task->{result} = $data;
 		$task->{attachmentref} = $resultattachmentref;
 		print "PID $pid $task->{result}->{console_out}\n";
+		my ($result) = $ft->finish_task($task);
+		delete $task_list->{$taskid};
+	
 		if ($task->{result}->{journal_entry}) {
 			confess unless defined $journal;
 			$journal->add_entry($task->{result}->{journal_entry});
 		}
-
-		delete $task_list->{$taskid};
-		my ($result) = $ft->finish_task($task);
-
+		  
 		if ($result eq 'done') {
 			return ($task->{result}, $task->{attachmentref});
-		}
+		} 
 	}
 	return 0;
 }
