@@ -20,8 +20,6 @@
 
 package App::MtAws::RetrieveCommand;
 
-our $VERSION = '0.975_05';
-
 use strict;
 use warnings;
 use utf8;
@@ -63,8 +61,7 @@ sub get_file_list
 	# TODO: refactor
 	my @filelist =
 		grep { !$j->{active_retrievals}{$_->{archive_id}} && ! -f binaryfilename $_->{filename} }
-		map { {archive_id => $_->{archive_id}, relfilename => $_->{relfilename}, filename=> $j->absfilename($_->{relfilename}) } }
-		map { $j->latest($_) } # TODO: two maps is not effective
+		map { {archive_id => $files->{$_}->{archive_id}, relfilename =>$_, filename=> $j->absfilename($_) } }
 		keys %{$files};
 	@filelist  = splice(@filelist, 0, $options->{'max-number-of-files'});
 }
