@@ -20,24 +20,23 @@
 
 package App::MtAws::SingleDownloadJob;
 
-our $VERSION = '0.975_05';
-
 use strict;
 use warnings;
 use utf8;
 use base qw/App::MtAws::Job/;
 use App::MtAws::Utils;
+use File::stat;
 use Carp;
 
 
 sub new
 {
-	my ($class, %args) = @_;
-	my $self = \%args;
-	bless $self, $class;
-	$self->{archive}||confess;
-	$self->{raised} = 0;
-	return $self;
+    my ($class, %args) = @_;
+    my $self = \%args;
+    bless $self, $class;
+    $self->{archive}||confess;
+    $self->{raised} = 0;
+    return $self;
 }
 
 # returns "ok" "wait" "ok subtask"
@@ -62,10 +61,11 @@ sub finish_task
 	my ($self, $task) = @_;
 	if ($self->{raised}) {
 		my $mtime = $task->{data}{mtime};
+		utime $mtime, $mtime, binaryfilename($task->{data}{filename}) or confess if defined $mtime;
 		return ("done");
 	} else {
 		confess;
 	}
 }
-
+	
 1;
