@@ -22,7 +22,7 @@ package App::MtAws::Utils;
 
 
 use strict;
-use warnings FATAL => 'all';
+use warnings;
 use utf8;
 use File::Spec;
 use Carp;
@@ -38,7 +38,7 @@ use base qw/Exporter/;
 
 our @EXPORT = qw/set_filename_encoding get_filename_encoding binaryfilename
 sanity_relative_filename is_relative_filename open_file sysreadfull syswritefull hex_dump_string
-is_wide_string/;
+is_wide_string characterfilename try_drop_utf8_flag/;
 
 # Does not work with directory names
 sub sanity_relative_filename
@@ -53,11 +53,15 @@ sub sanity_relative_filename
 	return $filename;
 }
 
-sub is_relative_filename # TODO: test
+sub is_relative_filename
 {
 	my ($filename) = @_;
-	my $newname = sanity_relative_filename($filename);
-	return defined($newname) && ($filename eq $newname); 
+	return unless (defined($filename) && length($filename));
+	return if $filename =~ tr{\r\n\t}{} or index($filename, '//') != -1 or substr($filename, 0, 1) eq '/';
+	return undef if $filename =~ m{
+		(^|/)\.\.?(/|$)
+	}x;
+	1;
 }
 
 
@@ -71,6 +75,10 @@ sub binaryfilename(;$)
 	encode(get_filename_encoding, @_ ? shift : $_, Encode::DIE_ON_ERR|Encode::LEAVE_SRC);	
 }
 
+sub characterfilename(;$)
+{
+	decode(get_filename_encoding, @_ ? shift : $_, Encode::DIE_ON_ERR|Encode::LEAVE_SRC);	
+}
 
 =pod
 
@@ -105,7 +113,7 @@ sub open_file($$%)
 	confess "Unknown argument(s) to open_file: ".join(';', keys %checkargs) if %checkargs;
 	
 	confess 'Argument "mode" is required' unless defined($args{mode});
-	confess "unknown mode $args{mode}" unless $args{mode} =~ m!^(<|>>?)$!;
+	confess "unknown mode $args{mode}" unless $args{mode} =~ m!^\+?(<|>>?)$!;
 	my $mode = $args{mode};
 	
 	confess "not_empty can be used in read mode only"
@@ -153,6 +161,13 @@ sub is_wide_string
 	defined($_[0]) && utf8::is_utf8($_[0]) && (bytes::length($_[0]) != length($_[0]))
 }
 
+# if we have ASCII-only data, let's drop UTF-8 flag in order to optimize some regexp stuff
+# TODO: write also version which does not check is_utf8 - it's faster when utf8 always set 
+sub try_drop_utf8_flag
+{
+	Encode::_utf8_off($_[0]) if utf8::is_utf8($_[0]) && (bytes::length($_[0]) == length($_[0]));
+}
+
 sub sysreadfull($$$)
 {
 	my ($file, $len) = ($_[0], $_[2]);
